@@ -164,8 +164,7 @@ class dynamic_pars(Intervention):
 
     **Examples**::
 
-        interv = cv.dynamic_pars({'diag_factor':{'days':30, 'vals':0.5}, 'cont_factor':{'days':30, 'vals':0.5}}) # Starting day 30, make diagnosed people and people with contacts half as likely to transmit
-        interv = cv.dynamic_pars({'beta':{'days':[14, 28], 'vals':[0.005, 0.015]}}) # On day 14, change beta to 0.005, and on day 28 change it back to 0.015
+        interv = cv.dynamic_pars({'beta':{'days':[14, 28], 'vals':[0.005, 0.015]}, 'diag_factor':{'days':30, 'vals':0.0}}) # Change beta, and make diagnosed people stop transmitting
     '''
 
     def __init__(self, pars, do_plot=None):
@@ -193,11 +192,11 @@ class dynamic_pars(Intervention):
         for parkey,parval in self.pars.items():
             inds = sc.findinds(parval['days'], t) # Look for matches
             if len(inds):
-
                 if len(inds)>1:
                     raise ValueError(f'Duplicate days are not allowed for Dynamic interventions (day={t}, indices={inds})')
                 else:
                     match = inds[0]
+                    self.days.append(t)
                     val = parval['vals'][match]
                     if isinstance(val, dict):
                         sim[parkey].update(val) # Set the parameter if a nested dict
@@ -211,7 +210,7 @@ class sequence(Intervention):
     This is an example of a meta-intervention which switches between a sequence of interventions.
 
     Args:
-        days (list): the days on which to apply each intervention
+        days (list): the days on which to start applying each intervention
         interventions (list): the interventions to apply on those days
         do_plot (bool): whether or not to plot the intervention
 
@@ -228,7 +227,6 @@ class sequence(Intervention):
         assert len(days) == len(interventions)
         self.days = days
         self.interventions = interventions
-        self._cum_days = np.cumsum(days)
         self._store_args()
         return
 
@@ -236,13 +234,16 @@ class sequence(Intervention):
     def initialize(self, sim):
         ''' Fix the dates '''
         self.days = [sim.day(day) for day in self.days]
+        self.days_arr = np.array(self.days + [sim.npts])
         self.initialized = True
         return
 
 
     def apply(self, sim):
-        idx = np.argmax(self._cum_days > sim.t)  # Index of the intervention to apply on this day
-        self.interventions[idx].apply(sim)
+        inds = sc.findinds(self.days_arr <= sim.t)
+        if len(inds):
+            idx = max(inds)  # Index of the intervention to apply on this day
+            self.interventions[idx].apply(sim)
         return
 
 
@@ -421,7 +422,7 @@ class test_num(Intervention):
     Test a fixed number of people per day.
 
     Args:
-        daily_tests (int or arr): number of tests per day
+        daily_tests (int or arr): number of tests per day; if integer, use that number every day
         symp_test (float): odds ratio of a symptomatic person testing
         quar_test (float): probability of a person in quarantine testing
         sensitivity (float): test sensitivity
@@ -439,10 +440,10 @@ class test_num(Intervention):
         Intervention
     '''
 
-    def __init__(self, daily_tests, sympt_test=100.0, quar_test=1.0, sensitivity=1.0, loss_prob=0, test_delay=0, start_day=0, end_day=None, do_plot=None):
+    def __init__(self, daily_tests, symp_test=100.0, quar_test=1.0, sensitivity=1.0, loss_prob=0, test_delay=0, start_day=0, end_day=None, do_plot=None):
         super().__init__(do_plot=do_plot)
         self.daily_tests = daily_tests #: Should be a list of length matching time
-        self.sympt_test  = sympt_test
+        self.symp_test  = symp_test
         self.quar_test   = quar_test
         self.sensitivity = sensitivity
         self.loss_prob   = loss_prob
@@ -454,11 +455,24 @@ class test_num(Intervention):
 
 
     def initialize(self, sim):
-        ''' Fix the dates '''
+        ''' Fix the dates and number of tests '''
+
+        # Handle days
         self.start_day   = sim.day(self.start_day)
         self.end_day     = sim.day(self.end_day)
         self.days        = [self.start_day, self.end_day]
+
+        # Process daily tests -- has to be here rather than init so have access to the sim object
+        if sc.isnumber(self.daily_tests): # If a number, convert to an array
+            self.daily_tests = np.array([int(self.daily_tests)]*sim.npts)
+        elif isinstance(self.daily_tests, (pd.Series, pd.DataFrame)):
+            start_date = sim['start_day'] + dt.timedelta(days=self.start_day)
+            end_date = self.daily_tests.index[-1]
+            dateindex = pd.date_range(start_date, end_date)
+            self.daily_tests = self.daily_tests.reindex(dateindex, fill_value=0).to_numpy()
+
         self.initialized = True
+
         return
 
 
@@ -470,17 +484,10 @@ class test_num(Intervention):
         elif self.end_day is not None and t > self.end_day:
             return
 
-        # Process daily tests -- has to be here rather than init so have access to the sim object
-        if isinstance(self.daily_tests, (pd.Series, pd.DataFrame)):
-            start_date = sim['start_day'] + dt.timedelta(days=self.start_day)
-            end_date = self.daily_tests.index[-1]
-            dateindex = pd.date_range(start_date, end_date)
-            self.daily_tests = self.daily_tests.reindex(dateindex, fill_value=0).to_numpy()
-
         # Check that there are still tests
         rel_t = t - self.start_day
         if rel_t < len(self.daily_tests):
-            n_tests = self.daily_tests[rel_t]  # Number of tests for this day
+            n_tests = int(self.daily_tests[rel_t]/sim.rescale_vec[t])  # Number of tests for this day -- rescaled
             if not (n_tests and pl.isfinite(n_tests)): # If there are no tests today, abort early
                 return
             else:
@@ -492,7 +499,7 @@ class test_num(Intervention):
         symp_inds  = cvu.true(sim.people.symptomatic)
         quar_inds  = cvu.true(sim.people.quarantined)
         diag_inds  = cvu.true(sim.people.diagnosed)
-        test_probs[symp_inds] *= self.sympt_test
+        test_probs[symp_inds] *= self.symp_test
         test_probs[quar_inds] *= self.quar_test
         test_probs[diag_inds] = 0.
 
@@ -520,13 +527,13 @@ class test_prob(Intervention):
 
     **Example**::
 
-        interv = cv.test_prob(symptomatic_prob=0.1, asymptomatic_prob=0.01) # Test 10% of symptomatics and 1% of asymptomatics
+        interv = cv.test_prob(symp_prob=0.1, asymp_prob=0.01) # Test 10% of symptomatics and 1% of asymptomatics
         interv = cv.test_prob(symp_quar_prob=0.4) # Test 40% of those in quarantine with symptoms
 
     Returns:
         Intervention
     '''
-    def __init__(self, symp_prob=0, asymp_prob=0, symp_quar_prob=None, asymp_quar_prob=None, test_sensitivity=1.0, loss_prob=0.0, test_delay=1, start_day=0, end_day=None, do_plot=None):
+    def __init__(self, symp_prob, asymp_prob=0.0, symp_quar_prob=None, asymp_quar_prob=None, test_sensitivity=1.0, loss_prob=0.0, test_delay=1, start_day=0, end_day=None, do_plot=None):
         super().__init__(do_plot=do_plot)
         self.symp_prob        = symp_prob
         self.asymp_prob       = asymp_prob
@@ -574,7 +581,7 @@ class test_prob(Intervention):
         test_inds = cvu.binomial_arr(test_probs).nonzero()[0]
 
         sim.people.test(test_inds, test_sensitivity=self.test_sensitivity, loss_prob=self.loss_prob, test_delay=self.test_delay)
-        sim.results['new_tests'][t] += len(test_inds)
+        sim.results['new_tests'][t] += int(len(test_inds)*sim['pop_scale']/sim.rescale_vec[t]) # If we're using dynamic scaling, we have to scale by pop_scale, not rescale_vec
 
         return
 
@@ -591,8 +598,12 @@ class contact_tracing(Intervention):
         contact_red (float): not implemented currently, but could potentially scale contact in this intervention
         do_plot (bool): whether or not to plot
     '''
-    def __init__(self, trace_probs, trace_time, start_day=0, end_day=None, contact_red=None, do_plot=None):
+    def __init__(self, trace_probs=None, trace_time=None, start_day=0, end_day=None, contact_red=None, do_plot=None):
         super().__init__(do_plot=do_plot)
+        if trace_probs is None:
+            trace_probs = 1.0
+        if trace_time is None:
+            trace_time = 0.0
         self.trace_probs = trace_probs
         self.trace_time  = trace_time
         self.contact_red = contact_red
