@@ -7,6 +7,7 @@ webapp.
 '''
 
 import numpy as np
+import pandas as pd
 import pylab as pl
 import sciris as sc
 import datetime as dt
@@ -15,7 +16,7 @@ import plotly.graph_objects as go
 from . import defaults as cvd
 
 
-__all__ = ['plot_sim', 'plot_scens', 'plot_result', 'plotly_sim', 'plotly_people', 'plotly_animate']
+__all__ = ['plot_sim', 'plot_scens', 'plot_result', 'plot_compare', 'plot_transtree', 'animate_transtree', 'plotly_sim', 'plotly_people', 'plotly_animate']
 
 
 #%% Plotting helper functions
@@ -200,7 +201,7 @@ def tidy_up(fig, figs, sep_figs, do_save, fig_path, do_show, default_name='covas
 def plot_sim(sim, to_plot=None, do_save=None, fig_path=None, fig_args=None, plot_args=None,
          scatter_args=None, axis_args=None, fill_args=None, legend_args=None, as_dates=True, dateformat=None,
          interval=None, n_cols=1, font_size=18, font_family=None, grid=False, commaticks=True, setylim=True,
-         log_scale=False, colors=None, do_show=True, sep_figs=False, fig=None):
+         log_scale=False, colors=None, labels=None, do_show=True, sep_figs=False, fig=None):
     ''' Plot the results of a single simulation -- see Sim.plot() for documentation '''
 
     # Handle inputs
@@ -218,9 +219,13 @@ def plot_sim(sim, to_plot=None, do_save=None, fig_path=None, fig_args=None, plot
                 color = colors[reskey]
             else:
                 color = res.color
+            if labels is not None:
+                label = labels[reskey]
+            else:
+                label = res.name
             if res.low is not None and res.high is not None:
                 ax.fill_between(res_t, res.low, res.high, color=color, **args.fill) # Create the uncertainty bound
-            ax.plot(res_t, res.values, label=res.name, **args.plot, c=color)
+            ax.plot(res_t, res.values, label=label, **args.plot, c=color)
             plot_data(sim, reskey, args.scatter) # Plot the data
             reset_ticks(ax, sim, interval, as_dates) # Optionally reset tick marks (useful for e.g. plotting weeks/months)
         plot_interventions(sim, ax) # Plot the interventions
@@ -232,7 +237,7 @@ def plot_sim(sim, to_plot=None, do_save=None, fig_path=None, fig_args=None, plot
 def plot_scens(scens, to_plot=None, do_save=None, fig_path=None, fig_args=None, plot_args=None,
          scatter_args=None, axis_args=None, fill_args=None, legend_args=None, as_dates=True, dateformat=None,
          interval=None, n_cols=1, font_size=18, font_family=None, grid=False, commaticks=True, setylim=True,
-         log_scale=False, colors=None, do_show=True, sep_figs=False, fig=None):
+         log_scale=False, colors=None, labels=None, do_show=True, sep_figs=False, fig=None):
     ''' Plot the results of a scenario -- see Scenarios.plot() for documentation '''
 
     # Handle inputs
@@ -253,8 +258,12 @@ def plot_scens(scens, to_plot=None, do_save=None, fig_path=None, fig_args=None,
                     color = colors[scenkey]
                 else:
                     color = default_colors[snum]
+                if labels is not None:
+                    label = labels[scenkey]
+                else:
+                    label = scendata.name
                 ax.fill_between(scens.tvec, scendata.low, scendata.high, color=color, **args.fill) # Create the uncertainty bound
-                ax.plot(scens.tvec, res_y, label=scendata.name, c=color, **args.plot) # Plot the actual line
+                ax.plot(scens.tvec, res_y, label=label, c=color, **args.plot) # Plot the actual line
                 plot_data(sim, reskey, args.scatter) # Plot the data
                 plot_interventions(sim, ax) # Plot the interventions
                 reset_ticks(ax, sim, interval, as_dates) # Optionally reset tick marks (useful for e.g. plotting weeks/months)
@@ -265,7 +274,7 @@ def plot_scens(scens, to_plot=None, do_save=None, fig_path=None, fig_args=None,
 
 def plot_result(sim, key, fig_args=None, plot_args=None, axis_args=None, scatter_args=None,
                 font_size=18, font_family=None, grid=False, commaticks=True, setylim=True,
-                as_dates=True, dateformat=None, interval=None, color=None, fig=None):
+                as_dates=True, dateformat=None, interval=None, color=None, label=None, fig=None):
     ''' Plot a single result -- see Sim.plot_result() for documentation '''
 
     # Handle inputs
@@ -291,7 +300,9 @@ def plot_result(sim, key, fig_args=None, plot_args=None, axis_args=None, scatter
         ax = pl.subplot(111, label='plot_result')
 
     # Do the plotting
-    ax.plot(res_t, res_y, c=color, label=res.name, **args.plot)
+    if label is None:
+        label = res.name
+    ax.plot(res_t, res_y, c=color, label=label, **args.plot)
     plot_data(sim, key, args.scatter) # Plot the data
     plot_interventions(sim, ax) # Plot the interventions
     title_grid_legend(ax, res.name, grid, commaticks, setylim, args.legend) # Configure the title, grid, and legend
@@ -300,6 +311,227 @@ def plot_result(sim, key, fig_args=None, plot_args=None, axis_args=None, scatter
     return fig
 
 
+def plot_compare(df, log_scale=True, fig_args=None, plot_args=None, axis_args=None, scatter_args=None,
+                font_size=18, font_family=None, grid=False, commaticks=True, setylim=True,
+                as_dates=True, dateformat=None, interval=None, color=None, label=None, fig=None):
+    ''' Plot a MultiSim comparison -- see MultiSim.plot_compare() for documentation '''
+
+    # Handle inputs
+    fig_args  = sc.mergedicts({'figsize':(16,16)}, fig_args)
+    axis_args = sc.mergedicts({'left': 0.16, 'bottom': 0.05, 'right': 0.98, 'top': 0.98, 'wspace': 0.50, 'hspace': 0.10}, axis_args)
+    args = handle_args(fig_args, plot_args, scatter_args, axis_args)
+    fig, figs, ax = create_figs(args, font_size, font_family, sep_figs=False, fig=fig)
+
+    # Map from results into different categories
+    mapping = {
+        'cum': 'Cumulative counts',
+        'new': 'New counts',
+        'n': 'Number in state',
+        'r': 'R_eff',
+        }
+    category = []
+    for v in df.index.values:
+        v_type = v.split('_')[0]
+        if v_type in mapping:
+            category.append(v_type)
+        else:
+            category.append('other')
+    df['category'] = category
+
+    # Plot
+    for i,m in enumerate(mapping):
+        not_r_eff = m != 'r'
+        if not_r_eff:
+            ax = fig.add_subplot(2, 2, i+1)
+        else:
+            ax = fig.add_subplot(8, 2, 10)
+        dfm = df[df['category'] == m]
+        logx = not_r_eff and log_scale
+        dfm.plot(ax=ax, kind='barh', logx=logx, legend=False)
+        if not(not_r_eff):
+            ax.legend(loc='upper left', bbox_to_anchor=(0,-0.3))
+        ax.grid(True)
+
+    return fig
+
+
+#%% Transtree functions
+def plot_transtree(tt, *args, **kwargs):
+    ''' Plot the transmission tree; see TransTree.plot() for documentation '''
+
+    fig_args = kwargs.get('fig_args', dict(figsize=(16,10)))
+
+    if tt.detailed is None:
+        errormsg = 'Please run sim.people.make_detailed_transtree() before calling plotting'
+        raise ValueError(errormsg)
+
+    ttlist = []
+    for entry in tt.detailed:
+        if entry and entry.source:
+            tdict = {
+                'date': entry.date,
+                'layer': entry.layer,
+                's_asymp': entry.s.is_asymp,
+                's_presymp': entry.s.is_presymp,
+                's_sev': entry.s.is_severe,
+                's_crit': entry.s.is_critical,
+                's_diag': entry.s.is_diagnosed,
+                's_quar': entry.s.is_quarantined,
+                't_quar': entry.t.is_quarantined,
+                     }
+            ttlist.append(tdict)
+
+    df = pd.DataFrame(ttlist).rename(columns={'date': 'Day'})
+    df = df.loc[df['layer'] != 'seed_infection']
+
+    df['Stage'] = 'Symptomatic'
+    df.loc[df['s_asymp'], 'Stage'] = 'Asymptomatic'
+    df.loc[df['s_presymp'], 'Stage'] = 'Presymptomatic'
+
+    df['Severity'] = 'Mild'
+    df.loc[df['s_sev'], 'Severity'] = 'Severe'
+    df.loc[df['s_crit'], 'Severity'] = 'Critical'
+
+    fig = pl.figure(**fig_args)
+    i=1; r=2; c=3
+
+    def plot(key, title, i):
+        dat = df.groupby(['Day', key]).size().unstack(key)
+        ax = pl.subplot(r,c,i);
+        dat.plot(ax=ax, legend=None)
+        pl.legend(title=None)
+        ax.set_title(title)
+
+    to_plot = {
+        'layer':'Layer',
+        'Stage':'Source stage',
+        's_diag':'Source diagnosed',
+        's_quar':'Source quarantined',
+        't_quar':'Target quarantined',
+        'Severity':'Symptomatic source severity'
+    }
+    for i, (key, title) in enumerate(to_plot.items()):
+        plot(key, title, i+1)
+
+    return fig
+
+
+def animate_transtree(tt, **kwargs):
+    ''' Plot an animation of the transmission tree; see TransTree.animate() for documentation '''
+
+    # Settings
+    animate    = kwargs.get('animate', True)
+    verbose    = kwargs.get('verbose', False)
+    msize      = kwargs.get('markersize', 10)
+    sus_color  = kwargs.get('sus_color', [0.5, 0.5, 0.5])
+    fig_args   = kwargs.get('fig_args', dict(figsize=(24,16)))
+    axis_args  = kwargs.get('axis_args', dict(left=0.10, bottom=0.05, right=0.85, top=0.97, wspace=0.25, hspace=0.25))
+    plot_args  = kwargs.get('plot_args', dict(lw=2, alpha=0.5))
+    delay      = kwargs.get('delay', 0.2)
+    font_size  = kwargs.get('font_size', 18)
+    colors     = kwargs.get('colors', None)
+    cmap       = kwargs.get('cmap', 'parula')
+    pl.rcParams['font.size'] = font_size
+    if colors is None:
+        colors = sc.vectocolor(tt.pop_size, cmap=cmap)
+
+    # Initialization
+    n = tt.n_days + 1
+    frames = [list() for i in range(n)]
+    tests  = [list() for i in range(n)]
+    diags  = [list() for i in range(n)]
+    quars  = [list() for i in range(n)]
+
+    # Construct each frame of the animation
+    for i,entry in enumerate(tt.detailed): # Loop over every person
+        frame = sc.objdict()
+        tdq = sc.objdict() # Short for "tested, diagnosed, or quarantined"
+
+        # This person became infected
+        if entry:
+            source = entry['source']
+            target = entry['target']
+            target_date = entry['date']
+            if source: # Seed infections and importations won't have a source
+                source_date = tt.detailed[source]['date']
+            else:
+                source = 0
+                source_date = 0
+
+            # Construct this frame
+            frame.x = [source_date, target_date]
+            frame.y = [source, target]
+            frame.c = colors[source]
+            frame.i = True # If this person is infected
+            frames[target_date].append(frame)
+
+            # Handle testing, diagnosis, and quarantine
+            tdq.t = target
+            tdq.d = target_date
+            tdq.c = colors[target]
+            date_t = entry.t.date_tested
+            date_d = entry.t.date_diagnosed
+            date_q = entry.t.date_known_contact
+            if ~np.isnan(date_t) and date_t<n: tests[int(date_t)].append(tdq)
+            if ~np.isnan(date_d) and date_d<n: diags[int(date_d)].append(tdq)
+            if ~np.isnan(date_q) and date_q<n: quars[int(date_q)].append(tdq)
+
+        # This person did not become infected
+        else:
+            frame.x = [0]
+            frame.y = [i]
+            frame.c = sus_color
+            frame.i = False
+            frames[0].append(frame)
+
+    # Configure plotting
+    fig = pl.figure(**fig_args)
+    pl.subplots_adjust(**axis_args)
+    ax = fig.add_subplot(1,1,1)
+
+    # Create the legend
+    ax2 = pl.axes([0.85, 0.05, 0.14, 0.9])
+    ax2.axis('off')
+    lcol = colors[0]
+    na = np.nan # Shorten
+    pl.plot(na, na, '-', c=lcol, **plot_args, label='Transmission')
+    pl.plot(na, na, 'o', c=lcol, markersize=msize, **plot_args, label='Source')
+    pl.plot(na, na, '*', c=lcol, markersize=msize, **plot_args, label='Target')
+    pl.plot(na, na, 'o', c=lcol, markersize=msize*2, fillstyle='none', **plot_args, label='Tested')
+    pl.plot(na, na, 's', c=lcol, markersize=msize*1.2, **plot_args, label='Diagnosed')
+    pl.plot(na, na, 'x', c=lcol, markersize=msize*2.0, label='Known contact')
+    pl.legend()
+
+    # Plot the animation
+    pl.sca(ax)
+    for day in range(n):
+        pl.title(f'Day: {day}')
+        pl.xlim([0, n])
+        pl.ylim([0, tt.pop_size])
+        pl.xlabel('Day')
+        pl.ylabel('Person')
+        flist = frames[day]
+        tlist = tests[day]
+        dlist = diags[day]
+        qlist = quars[day]
+        if verbose: print(i, flist)
+        for f in flist:
+            if verbose: print(f)
+            pl.plot(f.x[0], f.y[0], 'o', c=f.c, markersize=msize, **plot_args) # Plot sources
+            pl.plot(f.x, f.y, '-', c=f.c, **plot_args) # Plot transmission lines
+            if f.i: # If this person is infected
+                pl.plot(f.x[1], f.y[1], '*', c=f.c, markersize=msize, **plot_args) # Plot targets
+        for tdq in tlist: pl.plot(tdq.d, tdq.t, 'o', c=tdq.c, markersize=msize*2, fillstyle='none') # Tested; No alpha for this
+        for tdq in dlist: pl.plot(tdq.d, tdq.t, 's', c=tdq.c, markersize=msize*1.2, **plot_args) # Diagnosed
+        for tdq in qlist: pl.plot(tdq.d, tdq.t, 'x', c=tdq.c, markersize=msize*2.0) # Quarantine; no alpha for this
+        pl.plot([0, day], [0.5, 0.5], c='k', lw=5) # Plot the endless march of time
+        if animate: # Whether to animate
+            pl.pause(delay)
+
+    return fig
+
+
+
 #%% Plotly functions
 
 def get_individual_states(sim):
