@@ -99,7 +99,7 @@ class Sim(cvb.BaseSim):
         ''' Load the data to calibrate against, if provided '''
         self.datafile = datafile # Store this
         if datafile is not None: # If a data file is provided, load it
-            self.data = cvpar.load_data(filename=datafile, columns=datacols, **kwargs)
+            self.data = cvm.load_data(filename=datafile, columns=datacols, **kwargs)
 
         return
 
@@ -227,17 +227,19 @@ class Sim(cvb.BaseSim):
 
         dcols = cvd.get_colors() # Get default colors
 
+        # Flows and cumulative flows
+        for key,label in cvd.result_flows.items():
+            self.results[f'cum_{key}'] = init_res(f'Cumulative {label}',    color=dcols[key]) # Cumulative variables -- e.g. "Cumulative infections"
+
+        for key,label in cvd.result_flows.items(): # Repeat to keep all the cumulative keys together
+            self.results[f'new_{key}'] = init_res(f'Number of new {label}', color=dcols[key]) # Flow variables -- e.g. "Number of new infections"
+
         # Stock variables
         for key,label in cvd.result_stocks.items():
             self.results[f'n_{key}'] = init_res(label, color=dcols[key])
         self.results['n_susceptible'].scale = 'static'
         self.results['bed_capacity']  = init_res('Bed demand relative to capacity', scale=False)
 
-        # Flows and cumulative flows
-        for key,label in cvd.result_flows.items():
-            self.results[f'new_{key}'] = init_res(f'Number of new {label}', color=dcols[key]) # Flow variables -- e.g. "Number of new infections"
-            self.results[f'cum_{key}'] = init_res(f'Cumulative {label}',    color=dcols[key]) # Cumulative variables -- e.g. "Cumulative infections"
-
         # Other variables
         self.results['r_eff']         = init_res('Effective reproductive number', scale=False)
         self.results['doubling_time'] = init_res('Doubling time', scale=False)
@@ -319,6 +321,25 @@ class Sim(cvb.BaseSim):
         return
 
 
+    def rescale(self):
+        ''' Dynamically rescale the population '''
+        if self['rescale']:
+            t = self.t
+            pop_scale = self['pop_scale']
+            current_scale = self.rescale_vec[t]
+            if current_scale < pop_scale: # We have room to rescale
+                n_not_sus = self.people.count_not('susceptible')
+                n_people = len(self.people)
+                if n_not_sus / n_people > self['rescale_threshold']: # Check if we've reached point when we want to rescale
+                    max_ratio = pop_scale/current_scale # We don't want to exceed this
+                    scaling_ratio = min(self['rescale_factor'], max_ratio)
+                    self.rescale_vec[t:] *= scaling_ratio # Update the rescaling factor from here on
+                    n = int(n_people*(1.0-1.0/scaling_ratio)) # For example, rescaling by 2 gives n = 0.5*n_people
+                    new_sus_inds = cvu.choose(max_n=n_people, n=n) # Choose who to make susceptible again
+                    self.people.make_susceptible(new_sus_inds)
+        return
+
+
     def step(self):
         '''
         Step the simulation forward in time
@@ -334,7 +355,7 @@ class Sim(cvb.BaseSim):
         people   = self.people # Shorten this for later use
         flows    = people.update_states_pre(t=t) # Update the state of everyone and count the flows
         contacts = people.update_contacts() # Compute new contacts
-        bed_max  = people.count('severe') > self['n_beds'] # Check for a bed constraint
+        bed_max  = people.count('severe') > self['n_beds'] if self['n_beds'] else False # Check for a bed constraint
 
         # Randomly infect some people (imported infections)
         n_imports = cvu.poisson(self['n_imports']) # Imported cases
@@ -348,7 +369,7 @@ class Sim(cvb.BaseSim):
         for intervention in self['interventions']:
             intervention.apply(self)
         if self['interv_func'] is not None: # Apply custom intervention function
-            self =self['interv_func'](self)
+            self['interv_func'](self)
 
         flows = people.update_states_post(flows) # Check for state changes after interventions
 
@@ -365,36 +386,39 @@ class Sim(cvb.BaseSim):
         viral_load = cvu.compute_viral_load(t, date_inf, date_rec, date_dead, frac_time, load_ratio, high_cap)
 
         for lkey,layer in contacts.items():
-            sources = layer['p1']
-            targets = layer['p2']
+            p1 = layer['p1']
+            p2 = layer['p2']
             betas   = layer['beta']
 
             # Compute relative transmission and susceptibility
             rel_trans  = people.rel_trans
             rel_sus    = people.rel_sus
+            inf        = people.infectious
+            sus        = people.susceptible
             symp       = people.symptomatic
             diag       = people.diagnosed
             quar       = people.quarantined
             quar_eff   = cvd.default_float(self['quar_eff'][lkey])
             beta_layer = cvd.default_float(self['beta_layer'][lkey])
-            rel_trans, rel_sus = cvu.compute_trans_sus(rel_trans, rel_sus, beta_layer, viral_load, symp, diag, quar, asymp_factor, diag_factor, quar_eff)
+            rel_trans, rel_sus = cvu.compute_trans_sus(rel_trans, rel_sus, inf, sus, beta_layer, viral_load, symp, diag, quar, asymp_factor, diag_factor, quar_eff)
 
             # Calculate actual transmission
-            target_inds, edge_inds = cvu.compute_infections(beta, sources, targets, betas, rel_trans, rel_sus) # Calculate transmission!
-            flows['new_infections'] += people.infect(inds=target_inds, bed_max=bed_max) # Actually infect people
+            for sources,targets in [[p1,p2], [p2,p1]]: # Loop over the contact network from p1->p2 and p2->p1
+                target_inds, edge_inds = cvu.compute_infections(beta, sources, targets, betas, rel_trans, rel_sus) # Calculate transmission!
+                flows['new_infections'] += people.infect(inds=target_inds, bed_max=bed_max) # Actually infect people
 
-            # Store the transmission tree
-            for ind in edge_inds:
-                source = sources[ind]
-                target = targets[ind]
-                transdict = dict(source=source, target=target, date=self.t, layer=lkey)
-                self.people.transtree.linelist[target] = transdict
-                self.people.transtree.targets[source].append(transdict)
+                # Store the transmission tree
+                for ind in edge_inds:
+                    source = sources[ind]
+                    target = targets[ind]
+                    transdict = dict(source=source, target=target, date=self.t, layer=lkey)
+                    self.people.transtree.linelist[target] = transdict
+                    self.people.transtree.targets[source].append(transdict)
 
         # Update counts for this time step: stocks
         for key in cvd.result_stocks.keys():
             self.results[f'n_{key}'][t] = people.count(key)
-        self.results['bed_capacity'][t] = self.results['n_severe'][t]/self['n_beds'] if self['n_beds']>0 else np.nan
+        self.results['bed_capacity'][t] = self.results['n_severe'][t]/self['n_beds'] if self['n_beds'] else 0
 
         # Update counts for this time step: flows
         for key,count in flows.items():
@@ -405,25 +429,6 @@ class Sim(cvb.BaseSim):
         return
 
 
-    def rescale(self):
-        ''' Dynamically rescale the population '''
-        if self['rescale']:
-            t = self.t
-            pop_scale = self['pop_scale']
-            current_scale = self.rescale_vec[t]
-            if current_scale < pop_scale: # We have room to rescale
-                n_not_sus = self.people.count_not('susceptible')
-                n_people = len(self.people)
-                if n_not_sus / n_people > self['rescale_threshold']: # Check if we've reached point when we want to rescale
-                    max_ratio = pop_scale/current_scale # We don't want to exceed this
-                    scaling_ratio = min(self['rescale_factor'], max_ratio)
-                    self.rescale_vec[t+1:] *= scaling_ratio # Update the rescaling factor from here on
-                    n = int(n_people*(1.0-1.0/scaling_ratio)) # For example, rescaling by 2 gives n = 0.5*n_people
-                    new_sus_inds = cvu.choose(max_n=n_people, n=n) # Choose who to make susceptible again
-                    self.people.make_susceptible(new_sus_inds)
-        return
-
-
     def run(self, do_plot=False, until=None, verbose=None, **kwargs):
         '''
         Run the simulation.
@@ -447,6 +452,8 @@ class Sim(cvb.BaseSim):
             self.init_interventions() # And interventions
         if verbose is None:
             verbose = self['verbose']
+        if until:
+            until = self.day(until)
 
         # Main simulation loop
         for t in self.tvec:
@@ -641,7 +648,8 @@ class Sim(cvb.BaseSim):
         date_symptomatic = self.people.date_symptomatic
         for p in range(len(self.people)):
             if len(targets[p])>0:
-                for target_ind in targets[p]:
+                for target in targets[p]:
+                    target_ind = target['target']
                     intervals1[pos1] = date_exposed[target_ind] - date_exposed[p]
                     pos1 += 1
                     if not np.isnan(date_symptomatic[p]):
