@@ -1,11 +1,11 @@
 from unittest_support_classes import CovaSimTest
 from unittest_support_classes import TestProperties
 from math import sqrt
+import json
 import numpy as np
 
 import unittest
 
-import numpy as np
 
 ResultsKeys = TestProperties.ResultsDataKeys
 SimKeys = TestProperties.ParameterKeys.SimulationKeys
@@ -131,134 +131,217 @@ class InterventionTests(CovaSimTest):
                                    f"(with {total_infections[next_multiplier]} infections)")
 
     def test_change_beta_layers_clustered(self):
+        '''
+        Suggested alternative implementation:
+
+            import covasim as cv
+
+            # Define the interventions
+            days = dict(h=30, s=35, w=40, c=45)
+            interventions = []
+            for key,day in days.items():
+                interventions.append(cv.change_beta(days=day, changes=0, layers=key))
+
+            # Create and run the sim
+            sim = cv.Sim(pop_type='hybrid', n_days=60, interventions=interventions)
+            sim.run()
+            assert sim.results['new_infections'].values[days['c']:].sum() == 0
+            sim.plot()
+        '''
         self.is_debugging = False
         initial_infected = 10
-        params = {
-            SimKeys.number_agents: 5000,
-            SimKeys.number_simulated_days: 60,
-            SimKeys.initial_infected_count: initial_infected
-        }
-        self.set_simulation_parameters(params_dict=params)
-        day_of_change = 25
-        change_multipliers = [0.0]
-        layer_keys = ['c','h','s','w']
+        seed_list = range(0)
+        for seed in seed_list:
+            params = {
+                SimKeys.random_seed: seed,
+                SimKeys.number_agents: 5000,
+                SimKeys.number_simulated_days: 60,
+                SimKeys.initial_infected_count: initial_infected
+            }
+            if len(seed_list) > 1:
+                self.expected_result_filename = f"DEBUG_{self.id()}_{seed}.json"
+            self.set_simulation_parameters(params_dict=params)
+            day_of_change = 25
+            change_multipliers = [0.0]
+            layer_keys = ['c','h','s','w']
 
-        sequence_days = []
-        sequence_interventions = []
+            intervention_days = []
+            intervention_list = []
 
-        for k in layer_keys: # Zero out one layer at a time
-            day_of_change += 5
-            self.intervention_set_changebeta(
-                days_array=[day_of_change],
-                multiplier_array=change_multipliers,
-                layers=[k]
-            )
-            sequence_days.append(day_of_change)
-            sequence_interventions.append(self.interventions)
-            self.interventions = None
+            for k in layer_keys: # Zero out one layer at a time
+                day_of_change += 5
+                self.intervention_set_changebeta(
+                    days_array=[day_of_change],
+                    multiplier_array=change_multipliers,
+                    layers=[k]
+                )
+                intervention_days.append(day_of_change)
+                intervention_list.append(self.interventions)
+                self.interventions = None
+                pass
+            self.interventions = intervention_list
+            self.run_sim(population_type='clustered')
+            last_intervention_day = intervention_days[-1]
+            first_intervention_day = intervention_days[0]
+            cum_infections_channel= self.get_full_result_channel(ResultsKeys.infections_cumulative)
+            if len(seed_list) > 1:
+                messages = []
+                if cum_infections_channel[intervention_days[0]-1] < initial_infected:
+                    messages.append(f"Before intervention at day {intervention_days[0]}, there should be infections happening.")
+                    pass
+
+                if cum_infections_channel[last_intervention_day] < cum_infections_channel[first_intervention_day]:
+                    messages.append(f"Cumulative infections should grow with only some layers enabled.")
+                    pass
+
+                if cum_infections_channel[last_intervention_day] != cum_infections_channel[-1]:
+                    messages.append(f"The cumulative infections at {last_intervention_day} should be the same as at the end.")
+                    pass
+
+                if len(messages) > 0:
+                    print(f"ERROR: seed {seed}")
+                    for m in messages:
+                        print(f"\t{m}")
+                        pass
+
+            self.assertGreater(cum_infections_channel[intervention_days[0]-1],
+                               initial_infected,
+                               msg=f"Before intervention at day {intervention_days[0]}, there should be infections happening.")
+
+            self.assertGreater(cum_infections_channel[last_intervention_day],
+                               cum_infections_channel[first_intervention_day],
+                               msg=f"Cumulative infections should grow with only some layers enabled.")
+
+            self.assertEqual(cum_infections_channel[last_intervention_day],
+                             cum_infections_channel[-1],
+                             msg=f"with all layers at 0 beta, the cumulative infections at {last_intervention_day}" +
+                                 f" should be the same as at the end.")
             pass
-        self.intervention_build_sequence(day_list=sequence_days,
-                                         intervention_list=sequence_interventions)
-        self.run_sim(population_type='clustered')
-        last_intervention_day = sequence_days[-1]
-        first_intervention_day = sequence_days[0]
-        cum_infections_channel= self.get_full_result_channel(ResultsKeys.infections_cumulative)
-        self.assertGreater(cum_infections_channel[sequence_days[0]-1],
-                           initial_infected,
-                           msg=f"Before intervention at day {sequence_days[0]}, there should be infections happening.")
-        self.assertGreater(cum_infections_channel[last_intervention_day],
-                           cum_infections_channel[first_intervention_day],
-                           msg=f"Cumulative infections should grow with only some layers enabled.")
-        self.assertEqual(cum_infections_channel[last_intervention_day],
-                         cum_infections_channel[-1],
-                         msg=f"With all layers at 0 beta, the cumulative infections at {last_intervention_day}"
-                             f" should be the same as at the end.")
-        pass
 
     def test_change_beta_layers_random(self):
         self.is_debugging = False
         initial_infected = 10
-        params = {
-            SimKeys.number_agents: 5000,
-            SimKeys.number_simulated_days: 60,
-            SimKeys.initial_infected_count: initial_infected
-        }
-        self.set_simulation_parameters(params_dict=params)
-        day_of_change = 25
-        change_multipliers = [0.0]
-        layer_keys = ['a']
+        seed_list = range(0)
+        for seed in seed_list:
+            params = {
+                SimKeys.random_seed: seed,
+                SimKeys.number_agents: 5000,
+                SimKeys.number_simulated_days: 60,
+                SimKeys.initial_infected_count: initial_infected
+            }
+            self.set_simulation_parameters(params_dict=params)
+            if len(seed_list) > 1:
+                self.expected_result_filename = f"DEBUG_{self.id()}_{seed}.json"
+            day_of_change = 25
+            change_multipliers = [0.0]
+            layer_keys = ['a']
 
-        sequence_days = []
-        sequence_interventions = []
+            intervention_days = []
+            intervention_list = []
 
-        for k in layer_keys: # Zero out one layer at a time
-            day_of_change += 5
-            self.intervention_set_changebeta(
-                days_array=[day_of_change],
-                multiplier_array=change_multipliers,
-                layers=[k]
-            )
-            sequence_days.append(day_of_change)
-            sequence_interventions.append(self.interventions)
-            self.interventions = None
-            pass
-        self.intervention_build_sequence(day_list=sequence_days,
-                                         intervention_list=sequence_interventions)
-        self.run_sim(population_type='random')
-        last_intervention_day = sequence_days[-1]
-        cum_infections_channel= self.get_full_result_channel(ResultsKeys.infections_cumulative)
-        self.assertGreater(cum_infections_channel[sequence_days[0]-1],
-                           initial_infected,
-                           msg=f"Before intervention at day {sequence_days[0]}, there should be infections happening.")
-        self.assertEqual(cum_infections_channel[last_intervention_day],
-                         cum_infections_channel[sequence_days[0] - 1],
-                         msg=f"With all layers at 0 beta, should be 0 infections at {last_intervention_day}.")
+            for k in layer_keys: # Zero out one layer at a time
+                day_of_change += 5
+                self.intervention_set_changebeta(
+                    days_array=[day_of_change],
+                    multiplier_array=change_multipliers,
+                    layers=[k]
+                )
+                intervention_days.append(day_of_change)
+                intervention_list.append(self.interventions)
+                self.interventions = None
+                pass
+            self.interventions = intervention_list
+            self.run_sim(population_type='random')
+            last_intervention_day = intervention_days[-1]
+            cum_infections_channel = self.get_full_result_channel(ResultsKeys.infections_cumulative)
+            if len(seed_list) > 1:
+                messages = []
+                if cum_infections_channel[intervention_days[0]-1] < initial_infected:
+                    messages.append(f"Before intervention at day {intervention_days[0]}, there should be infections happening.")
+                    pass
+
+                if cum_infections_channel[last_intervention_day] != cum_infections_channel[-1]:
+                    messages.append(f"The cumulative infections at {last_intervention_day} should be the same as at the end.")
+                    pass
+
+                if len(messages) > 0:
+                    print(f"ERROR: seed {seed}")
+                    for m in messages:
+                        print(f"\t{m}")
+                        pass
+            self.assertGreater(cum_infections_channel[intervention_days[0]-1],
+                               initial_infected,
+                               msg=f"Before intervention at day {intervention_days[0]}, there should be infections happening.")
+            self.assertEqual(cum_infections_channel[last_intervention_day],
+                             cum_infections_channel[intervention_days[0] - 1],
+                             msg=f"With all layers at 0 beta, should be 0 infections at {last_intervention_day}.")
 
     def test_change_beta_layers_hybrid(self):
         self.is_debugging = False
         initial_infected = 10
-        params = {
-            SimKeys.number_agents: 5000,
-            SimKeys.number_simulated_days: 60,
-            SimKeys.initial_infected_count: initial_infected
-        }
-        self.set_simulation_parameters(params_dict=params)
-        day_of_change = 25
-        change_multipliers = [0.0]
-        layer_keys = ['c','s','w','h']
+        seed_list = range(0)
+        for seed in seed_list:
+            params = {
+                SimKeys.random_seed: seed,
+                SimKeys.number_agents: 5000,
+                SimKeys.number_simulated_days: 60,
+                SimKeys.initial_infected_count: initial_infected
+            }
+            if len(seed_list) > 1:
+                self.expected_result_filename = f"DEBUG_{self.id()}_{seed}.json"
+            self.set_simulation_parameters(params_dict=params)
+            day_of_change = 25
+            change_multipliers = [0.0]
+            layer_keys = ['c','s','w','h']
 
-        sequence_days = []
-        sequence_interventions = []
-        current_layers = []
+            intervention_days = []
+            intervention_list = []
 
-        for k in layer_keys: # Zero out one layer at a time
-            day_of_change += 5
-            current_layers.append(k)
-            self.intervention_set_changebeta(
-                days_array=[day_of_change],
-                multiplier_array=change_multipliers,
-                layers=current_layers
-            )
-            sequence_days.append(day_of_change)
-            sequence_interventions.append(self.interventions)
-            self.interventions = None
-            pass
-        self.intervention_build_sequence(day_list=sequence_days,
-                                         intervention_list=sequence_interventions)
-        self.run_sim(population_type='hybrid')
-        last_intervention_day = sequence_days[-1]
-        first_intervention_day = sequence_days[0]
-        cum_infections_channel= self.get_full_result_channel(ResultsKeys.infections_cumulative)
-        self.assertGreater(cum_infections_channel[sequence_days[0]-1],
-                           initial_infected,
-                           msg=f"Before intervention at day {sequence_days[0]}, there should be infections happening.")
-        self.assertGreater(cum_infections_channel[last_intervention_day],
-                           cum_infections_channel[first_intervention_day],
-                           msg=f"Cumulative infections should grow with only some layers enabled.")
-        self.assertEqual(cum_infections_channel[last_intervention_day],
-                         cum_infections_channel[-1],
-                         msg=f"With all layers at 0 beta, the cumulative infections at {last_intervention_day}"
-                             f" should be the same as at the end.")
+            for k in layer_keys: # Zero out one layer at a time
+                day_of_change += 5
+                self.intervention_set_changebeta(
+                    days_array=[day_of_change],
+                    multiplier_array=change_multipliers,
+                    layers=[k]
+                )
+                intervention_days.append(day_of_change)
+                intervention_list.append(self.interventions)
+                self.interventions = None
+                pass
+            self.interventions = intervention_list
+            self.run_sim(population_type='hybrid')
+            last_intervention_day = intervention_days[-1]
+            first_intervention_day = intervention_days[0]
+            cum_infections_channel = self.get_full_result_channel(ResultsKeys.infections_cumulative)
+            if len(seed_list) > 1:
+                messages = []
+                if cum_infections_channel[intervention_days[0]-1] < initial_infected:
+                    messages.append(f"Before intervention at day {intervention_days[0]}, there should be infections happening.")
+                    pass
+
+                if cum_infections_channel[last_intervention_day] < cum_infections_channel[first_intervention_day]:
+                    messages.append(f"Cumulative infections should grow with only some layers enabled.")
+                    pass
+
+                if cum_infections_channel[last_intervention_day] != cum_infections_channel[-1]:
+                    messages.append(f"The cumulative infections at {last_intervention_day} should be the same as at the end.")
+                    pass
+
+                if len(messages) > 0:
+                    print(f"ERROR: seed {seed}")
+                    for m in messages:
+                        print(f"\t{m}")
+                        pass
+            self.assertGreater(cum_infections_channel[intervention_days[0]-1],
+                               initial_infected,
+                               msg=f"Before intervention at day {intervention_days[0]}, there should be infections happening.")
+            self.assertGreater(cum_infections_channel[last_intervention_day],
+                               cum_infections_channel[first_intervention_day],
+                               msg=f"Cumulative infections should grow with only some layers enabled.")
+            self.assertEqual(cum_infections_channel[last_intervention_day],
+                             cum_infections_channel[-1],
+                             msg=f"With all layers at 0 beta, the cumulative infections at {last_intervention_day}"
+                                 f" should be the same as at the end.")
 
     @unittest.skip("TODO: re-enable when synthpops is guaranteed to be here")
     def test_change_beta_layers_synthpops(self):
@@ -315,7 +398,7 @@ class InterventionTests(CovaSimTest):
                                  target_pop_new_channel,
                                  target_test_count_channel=None):
         if not target_test_count_channel:
-            target_test_count = target_pop_count_channel
+            target_test_count = target_pop_count_channel # CK: unused
         if test_sensitivity < 1.0:
             raise ValueError("This test method only works with perfect test "
                              f"sensitivity. {test_sensitivity} won't cut it.")
@@ -481,61 +564,92 @@ class InterventionTests(CovaSimTest):
                                       target_pop_new_channel=None)
         pass
 
-    def test_test_prob_sensitivity(self, subtract_today_recoveries=True):
+    def test_test_prob_sensitivity(self, subtract_today_recoveries=False):
         self.is_debugging = False
-        params = {
-            SimKeys.number_agents: 5000,
-            SimKeys.number_simulated_days: 31
-        }
-        self.set_simulation_parameters(params_dict=params)
+        seed_list = range(0)
+        error_seeds = {}
+        for seed in seed_list:
+            params = {
+                SimKeys.random_seed: seed,
+                SimKeys.number_agents: 5000,
+                SimKeys.number_simulated_days: 31
+            }
+            self.set_simulation_parameters(params_dict=params)
 
-        symptomatic_probability_of_test = 1.0
-        test_sensitivities = [0.9, 0.7, 0.6, 0.2]
-        test_delay = 0
-        start_day = 30
+            symptomatic_probability_of_test = 1.0
+            test_sensitivities = [0.9, 0.7, 0.6, 0.2]
+            test_delay = 0
+            start_day = 30
 
-        for sensitivity in test_sensitivities:
-            self.intervention_set_test_prob(symptomatic_prob=symptomatic_probability_of_test,
-                                            test_sensitivity=sensitivity,
-                                            test_delay=test_delay,
-                                            start_day=start_day)
-            self.run_sim()
-            first_day_diagnoses = self.get_full_result_channel(
-                channel=ResultsKeys.diagnoses_at_timestep
-            )[start_day]
-            target_count = self.get_full_result_channel(
-                channel=ResultsKeys.symptomatic_at_timestep
-            )[start_day]
-            if subtract_today_recoveries:
-                recoveries_today = self.get_full_result_channel(
-                    channel=ResultsKeys.recovered_at_timestep
+            for sensitivity in test_sensitivities:
+                self.intervention_set_test_prob(symptomatic_prob=symptomatic_probability_of_test,
+                                                test_sensitivity=sensitivity,
+                                                test_delay=test_delay,
+                                                start_day=start_day)
+                self.run_sim()
+                first_day_diagnoses = self.get_full_result_channel(
+                    channel=ResultsKeys.diagnoses_at_timestep
                 )[start_day]
-                target_count = target_count - recoveries_today
-            ideal_diagnoses = target_count * sensitivity
+                target_count = self.get_full_result_channel(
+                    channel=ResultsKeys.symptomatic_at_timestep
+                )[start_day]
+                if subtract_today_recoveries:
+                    recoveries_today = self.get_full_result_channel(
+                        channel=ResultsKeys.recovered_at_timestep
+                    )[start_day]
+                    target_count = target_count - recoveries_today
+                ideal_diagnoses = target_count * sensitivity
 
-            standard_deviation = sqrt(sensitivity * (1 - sensitivity) * target_count)
-            # 95% confidence interval
-            min_tolerable_diagnoses = ideal_diagnoses - 2 * standard_deviation
-            max_tolerable_diagnoses = ideal_diagnoses + 2 * standard_deviation
+                standard_deviation = sqrt(sensitivity * (1 - sensitivity) * target_count)
+                # 95% confidence interval
+                min_tolerable_diagnoses = ideal_diagnoses - 2 * standard_deviation
+                max_tolerable_diagnoses = ideal_diagnoses + 2 * standard_deviation
 
-            if self.is_debugging:
-                print(f"\tMax: {max_tolerable_diagnoses} \n"
-                      f"\tMin: {min_tolerable_diagnoses} \n"
-                      f"\tTarget: {target_count} \n"
-                      f"\tPrevious day Target: {self.get_full_result_channel(channel=ResultsKeys.symptomatic_at_timestep)[start_day -1 ]} \n"
-                      f"\tSensitivity: {sensitivity} \n"
-                      f"\tIdeal: {ideal_diagnoses} \n"
-                      f"\tActual diagnoses: {first_day_diagnoses}\n")
-            self.assertGreaterEqual(first_day_diagnoses, min_tolerable_diagnoses,
-                                    msg=f"Expected at least {min_tolerable_diagnoses} diagnoses with {target_count}"
-                                        f" symptomatic and {sensitivity} sensitivity. Got {first_day_diagnoses}"
-                                        f" diagnoses, which is too low.")
-            self.assertLessEqual(first_day_diagnoses, max_tolerable_diagnoses,
-                                 msg=f"Expected no more than {max_tolerable_diagnoses} diagnoses with {target_count}"
-                                     f" symptomatic and {sensitivity} sensitivity. Got {first_day_diagnoses}"
-                                     f" diagnoses, which is too high.")
+                if self.is_debugging:
+                    print(f"\tMax: {max_tolerable_diagnoses} \n"
+                          f"\tMin: {min_tolerable_diagnoses} \n"
+                          f"\tTarget: {target_count} \n"
+                          f"\tPrevious day Target: {self.get_full_result_channel(channel=ResultsKeys.symptomatic_at_timestep)[start_day -1 ]} \n"
+                          f"\tSensitivity: {sensitivity} \n"
+                          f"\tIdeal: {ideal_diagnoses} \n"
+                          f"\tActual diagnoses: {first_day_diagnoses}\n")
+                    pass
+
+                too_low_message = f"Expected at least {min_tolerable_diagnoses} diagnoses" \
+                                  f" with {target_count} symptomatic and {sensitivity}" \
+                                  f" sensitivity. Got {first_day_diagnoses} diagnoses," \
+                                  f" which is too low."
+                too_high_message = f"Expected no more than {max_tolerable_diagnoses} diagnoses" \
+                                   f" with {target_count} symptomatic and {sensitivity}" \
+                                   f" sensitivity. Got {first_day_diagnoses} diagnoses, which" \
+                                   f" is too high."
+
+                if len(seed_list) > 1:
+                    local_errors = {}
+                    if first_day_diagnoses + 1 < min_tolerable_diagnoses:
+                        local_errors[sensitivity] = (f"LOW: {too_low_message} seed: {seed}")
+                    elif first_day_diagnoses - 1 > max_tolerable_diagnoses:
+                        local_errors[sensitivity] = (f"HIGH: {too_high_message} seed: {seed}")
+                    if len(local_errors) > 0:
+                        if seed not in error_seeds:
+                            error_seeds[seed] = local_errors
+                        else:
+                            error_seeds[seed][sensitivity] = local_errors[sensitivity]
+                else:
+                    self.assertGreaterEqual(first_day_diagnoses + 1, min_tolerable_diagnoses,
+                                            msg=too_low_message)
+                    self.assertLessEqual(first_day_diagnoses - 1, max_tolerable_diagnoses,
+                                         msg=too_high_message)
+                pass
             pass
-        pass
+        if len(seed_list) > 1:
+            with open(f"DEBUG_test_prob_sensitivity_sweep.json",'w') as outfile:
+                json.dump(error_seeds, outfile, indent=4)
+                pass
+            acceptable_losses = len(seed_list) // 10
+            self.assertLessEqual(len(error_seeds),
+                                 acceptable_losses,
+                                 msg=error_seeds)
 
     def test_test_prob_symptomatic_prob_of_test(self):
         params = {
@@ -661,7 +775,7 @@ class InterventionTests(CovaSimTest):
         sequence_days = [30, 40]
         sequence_interventions = []
 
-        layer_to_trace = 's'
+        layer_to_trace = 's' # CK: unused
         layers_to_zero_beta = ['c','h','w']
 
         self.intervention_set_test_prob(symptomatic_prob=1.0,
@@ -720,3 +834,7 @@ class InterventionTests(CovaSimTest):
         # TODO: loop through the layers and reproduce the school test above
         pass
     # endregion
+
+
+if __name__ == '__main__':
+    unittest.main()
